@@ -1,5 +1,6 @@
 # Copyright 2008-2009 WebDriver committers
 # Copyright 2008-2009 Google Inc.
+# Copyright 2013 BrowserStack
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,14 +17,13 @@
 import logging
 import socket
 import string
+import base64
+import httplib
+import datetime
+import urllib2 as url_request
 
 try:
-    from urllib import request as url_request
-except ImportError:
-    import urllib2 as url_request
-
-try:
-    from urllib import parse
+    from urllib2 import parse
 except ImportError:
     import urlparse as parse
 
@@ -33,7 +33,6 @@ from . import utils
 
 LOGGER = logging.getLogger(__name__)
 
-
 class Request(url_request.Request):
     """
     Extends the url_request.Request to support all HTTP request types.
@@ -133,13 +132,14 @@ class RemoteConnection(object):
     Communicates with the server using the WebDriver wire protocol:
     http://code.google.com/p/selenium/wiki/JsonWireProtocol
     """
-
     def __init__(self, remote_server_addr):
         # Attempt to resolve the hostname and get an IP address.
         parsed_url = parse.urlparse(remote_server_addr)
+        addr = ""
         if parsed_url.hostname:
             try:
                 netloc = socket.gethostbyname(parsed_url.hostname)
+                addr = netloc
                 if parsed_url.port:
                     netloc += ':%d' % parsed_url.port
                 if parsed_url.username:
@@ -155,6 +155,7 @@ class RemoteConnection(object):
                             parsed_url.hostname)
 
         self._url = remote_server_addr
+        self._conn = httplib.HTTPConnection(str(addr),str(parsed_url.port))
         self._commands = {
             Command.STATUS: ('GET', '/status'),
             Command.NEW_SESSION: ('POST', '/session'),
@@ -201,14 +202,10 @@ class RemoteConnection(object):
                 ('GET', '/session/$sessionId/element/$id/selected'),
             Command.SET_ELEMENT_SELECTED:
                 ('POST', '/session/$sessionId/element/$id/selected'),
-            Command.TOGGLE_ELEMENT:
-                ('POST', '/session/$sessionId/element/$id/toggle'),
             Command.IS_ELEMENT_ENABLED:
                 ('GET', '/session/$sessionId/element/$id/enabled'),
             Command.IS_ELEMENT_DISPLAYED:
                 ('GET', '/session/$sessionId/element/$id/displayed'),
-            Command.HOVER_OVER_ELEMENT:
-                ('POST', '/session/$sessionId/element/$id/hover'),
             Command.GET_ELEMENT_LOCATION:
                 ('GET', '/session/$sessionId/element/$id/location'),
             Command.GET_ELEMENT_LOCATION_ONCE_SCROLLED_INTO_VIEW:
@@ -228,10 +225,6 @@ class RemoteConnection(object):
             Command.SWITCH_TO_FRAME: ('POST', '/session/$sessionId/frame'),
             Command.SWITCH_TO_WINDOW: ('POST', '/session/$sessionId/window'),
             Command.CLOSE: ('DELETE', '/session/$sessionId/window'),
-            Command.DRAG_ELEMENT:
-                ('POST', '/session/$sessionId/element/$id/drag'),
-            Command.GET_SPEED: ('GET', '/session/$sessionId/speed'),
-            Command.SET_SPEED: ('POST', '/session/$sessionId/speed'),
             Command.GET_ELEMENT_VALUE_OF_CSS_PROPERTY:
                 ('GET',  '/session/$sessionId/element/$id/css/$propertyName'),
             Command.IMPLICIT_WAIT:
@@ -308,7 +301,7 @@ class RemoteConnection(object):
             Command.GET_LOCAL_STORAGE_ITEM:
                 ('GET', '/session/$sessionId/local_storage/key/$key'),
             Command.REMOVE_LOCAL_STORAGE_ITEM:
-                ('POST', '/session/$sessionId/local_storage/key/$key'),
+                ('DELETE', '/session/$sessionId/local_storage/key/$key'),
             Command.GET_LOCAL_STORAGE_KEYS:
                 ('GET', '/session/$sessionId/local_storage'),
             Command.SET_LOCAL_STORAGE_ITEM:
@@ -335,6 +328,8 @@ class RemoteConnection(object):
                 ('GET','/session/$sessionId/log/types'),
             }
 
+
+
     def execute(self, command, params):
         """
         Send a command to the remote server.
@@ -369,47 +364,40 @@ class RemoteConnection(object):
         LOGGER.debug('%s %s %s' % (method, url, data))
 
         parsed_url = parse.urlparse(url)
-        password_manager = None
+        headers = {}
+        headers["Connection"] = "Keep-Alive"
+        headers[method] = parsed_url.path
+        headers["User-Agent"] = "Python http auth"
+        headers["Content-type"] = "application/json;charset=\"UTF-8\""
+        headers["Accept"] = "application/json"
+        headers["Connection"] = "keep-alive"
+
+        # for basic auth
         if parsed_url.username:
-            netloc = parsed_url.hostname
-            if parsed_url.port:
-                netloc += ":%s" % parsed_url.port
-            cleaned_url = parse.urlunparse((parsed_url.scheme,
-                                               netloc,
-                                               parsed_url.path,
-                                               parsed_url.params,
-                                               parsed_url.query,
-                                               parsed_url.fragment))
-            password_manager = url_request.HTTPPasswordMgrWithDefaultRealm()
-            password_manager.add_password(None,
-                                          "%s://%s" % (parsed_url.scheme, netloc),
-                                          parsed_url.username,
-                                          parsed_url.password)
-            request = Request(cleaned_url, data=data.encode('utf-8'), method=method)
-        else:
-            request = Request(url, data=data.encode('utf-8'), method=method)
+            auth = base64.standard_b64encode('%s:%s' % (parsed_url.username, parsed_url.password)).replace('\n','')
+            # Authorization header
+            headers["Authorization"] = "Basic %s" % auth
 
-        request.add_header('Accept', 'application/json')
-        request.add_header('Content-Type', 'application/json;charset=UTF-8')
-
-        if password_manager:
-            opener = url_request.build_opener(url_request.HTTPRedirectHandler(),
-                                          HttpErrorHandler(),
-                                          url_request.HTTPBasicAuthHandler(password_manager))
-        else:
-            opener = url_request.build_opener(url_request.HTTPRedirectHandler(),
-                                          HttpErrorHandler())
-        response = opener.open(request)
+        self._conn.request(method, parsed_url.path, data, headers)
+        resp = self._conn.getresponse()
+        statuscode = resp.status
+        statusmessage = resp.msg
+        LOGGER.debug('%s %s' %(statuscode, statusmessage))
+        data = resp.read()
         try:
-            if response.code > 399 and response.code < 500:
-                return {'status': response.code, 'value': response.read()}
-            body = response.read().decode('utf-8').replace('\x00', '').strip()
-            content_type = [value for name, value in response.info().items() if name.lower() == "content-type"]
+            if statuscode > 399 and statuscode < 500:
+                return {'status': statuscode, 'value': data}
+            if statuscode >= 300 and statuscode < 304:
+                return self._request(resp.getheader('location'), method='GET')
+            body = data.decode('utf-8').replace('\x00', '').strip()
+            content_type = []
+            if resp.getheader('Content-Type') is not None:
+                content_type = resp.getheader('Content-Type').split(';')
             if not any([x.startswith('image/png') for x in content_type]):
                 try:
                     data = utils.load_json(body.strip())
                 except ValueError:
-                    if response.code > 199 and response.code < 300:
+                    if statuscode > 199 and statuscode < 300:
                         status = ErrorCode.SUCCESS
                     else:
                         status = ErrorCode.UNKNOWN_ERROR
@@ -428,4 +416,5 @@ class RemoteConnection(object):
                 data = {'status': 0, 'value': body.strip()}
                 return data
         finally:
-            response.close()
+            LOGGER.debug("Finished Request")
+            resp.close()
